#include "EEPROM_24FC1025.h"

void EEPROM_24FC1025_WriteSingle(byte control_byte, word address, byte data)
{
    // Send start puls
    I2C_Start();
    // Send control byte
    I2C_Wr(control_byte & ~WRITE); // set write mode to avoid error
    // Send higher 8 bits of 16 bits address
    I2C_Wr((byte)(address >> 8));
    // Send lower 8 bits of 16 bits address
    I2C_Wr((byte)address);
    // Send byte data
    I2C_Wr(data);

    // Do nothing while IC is busy
    while(!I2C_Is_Idle())
        asm nop;

    // Send stop puls
    I2C_Stop();

    // 20ms pause
    Delay_ms(20);
}

void EEPROM_24FC1025_WriteMultiple(byte control_byte, word address, word bytes2write, byte *data_buffer)
{
    word count = 0;

    I2C_Start();
    I2C_Wr(control_byte & ~WRITE);
    I2C_Wr((byte)(address >> 8));
    I2C_Wr((byte)address);

    for (count = 0; count < bytes2write && count < 128; count++)
    {
        I2C_Wr(data_buffer[count]);
        Delay_ms(20);
    }

    while(!I2C_Is_Idle())
        asm nop;

    I2C_Stop();

    Delay_ms(20);
}

byte EEPROM_24FC1025_ReadSingle(byte control_byte, word address)
{
    byte data = 0;

    I2C_Start();
    // Send control byte with write mode
    I2C_Wr(control_byte & ~WRITE);
    I2C_Wr((byte)(address >> 8));
    I2C_Wr((byte)address);
    // Send start puls again !
    I2C_Repeated_Start();
    // Send control byte with read mode
    I2C_Wr(control_byte | READ);
    // Read byte and send NO ACKNOWLEDGE puls (the '0')
    data = I2C_Rd(NO_ACKNOWLEDGE);

    while(!I2C_Is_Idle())
        asm nop;

    I2C_Stop();
    Delay_ms(20);

    return data;
}

void EEPROM_24FC1025_ReadMultiple(byte control_byte, word address, word bytes2read, byte *data_buffer)
{
    word count = 0;

    I2C_Start();
    I2C_Wr(control_byte & ~WRITE);
    I2C_Wr((byte)(address >> 8));
    I2C_Wr((byte)address);
    I2C_Repeated_Start();
    I2C_Wr(control_byte | READ);

    for (count = 0; count < (bytes2read - 1) && count <= 0xFFFF; count++)
    {
        // Read byte and send ACKNOWLEDGE puls
        data_buffer[count] = I2C_Rd(ACKNOWLEDGE);
        Delay_ms(20);
    }
    // Read last byte and send NO ACKNOWLEDGE puls
    data_buffer[count] = I2C_Rd(NO_ACKNOWLEDGE);

    while(!I2C_Is_Idle())
        asm nop;

    I2C_Stop();

    Delay_ms(20);
}
